#include <xsTypes.h>
#include <functionMap.h>
#include <stlToCArrays.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>
#include <cstring>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB10(DOTBVABS,dotbvabs,FLOATV,INT,FLOATV,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV)
#define DOTBVABS(ear,ne,param,ifl,photar,photer,siggas,siggrains,sigmol,sigavg) \
           CCALLSFSUB10(DOTBVABS,dotbvabs,FLOATV,INT,FLOATV,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV, \
             ear,ne,param,ifl,photar,photer,siggas,siggrains,sigmol,sigavg)
             
             

void tbvabs(const RealArray& energyArray, const RealArray& params,
        int spectrumNumber, RealArray& flux, RealArray& fluxErr, 
        const string& initString)
{
   // Wrapper routine for the Tuebingen-Boulder ISM code. Grabs the dynamic 
   // memory required.
   
   // Translated from tbvabs.f 03/09 (CG)
   
   using namespace XSutility;
   
   const int ne = static_cast<int>(energyArray.size() - 1);
   auto_array_ptr<float> apSiggas(new float[ne]);
   auto_array_ptr<float> apSiggrains(new float[ne]);
   auto_array_ptr<float> apSigmol(new float[ne]);
   auto_array_ptr<float> apSigavg(new float[ne]);
   
   float *ear=0, *pars=0, *photar=0, *photer=0;
   XSFunctions::stlToFloatArrays<float>(energyArray, params, flux, fluxErr,
           ear, pars, photar, photer);
   auto_array_ptr<float> apEar(ear);
   auto_array_ptr<float> apPars(pars);
   auto_array_ptr<float> apPhotar(photar);
   auto_array_ptr<float> apPhoter(photer);
   
   DOTBVABS(ear, ne, pars, spectrumNumber, photar, photer, apSiggas.get(),
      apSiggrains.get(), apSigmol.get(), apSigavg.get());
   
   XSFunctions::floatFluxToStl<float>(photar, photer, ne, false, flux, fluxErr);       

}

double fggrain(const char* element)
{
   //     Returns the grain composition for the grain models summarized
   //     by Wilms, Allen, McCray, 2000, ApJ, in press
   //     The value returned is the ratio of the GAS abundance to the total
   //     ISM abundance (the "depletion factor" commonly used by the ISM
   //     people).
   //
   // Arguments :
   //      element    C    i: The standard chemical element abbreviation
   //      fggrain    R    r: the deplection factor
   //
   // NOTE: This subroutine knows about more elements than the standard
   //     ones used in XSPEC. Therefore, NELTS=21!
   //
   // Version 1.0: Joern Wilms, IAA Tuebingen, Astronomie
   //    wilms@astro.uni-tuebingen.de

   // Translated from tbvabs.f 03/09 (CG)
   
   const int NELTS = 21;
   const char *elts[] =   {"H ", "He", "C ", "N ", "O ", "Ne", "Na", 
               "Mg", "Al", "Si", "P ", "S ", "Cl", "Ar", "Ca", "Ti",
               "Cr", "Mn", "Fe", "Co", "Ni"};
   const double fdep[] =  {1.0,  1.0,  0.5,  1.0,  0.6,   1.0,  0.25, 
               0.2,  0.02, 0.1,  0.6,  0.6,  0.5,  1.0,  0.003, 0.002,
               0.03, 0.07, 0.3,  0.05, 0.04};
   
   double val = 0.0;            
   for (int i=0; i<NELTS; ++i)
   {
      if (strcmp(elts[i], element) == 0)
      {
         val = fdep[i];
         break;
      }
   }
   return val;
}
